﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using Microsoft.SharePoint;
using Microsoft.SharePoint.Administration;
using Microsoft.Office.Word.Server.Conversions;
using Microsoft.Office.Word.Server.Service;
using System.IO;

namespace Kaboodle.SynchConverter
{
    public static class ExtensionMethods
    {

        #region SPFarm extension methods

        /// <summary>
        /// Returns the first WordServiceApplicationProxy in the farm or null if no proxy is found
        /// </summary>
        /// <param name="Farm"></param>
        /// <returns>WordServiceApplicationProxy</returns>
        public static WordServiceApplicationProxy WordServiceAppProxy(this SPFarm Farm)
        {
            var WordServiceAppProxyQry = (from SPServiceProxy SvcProxy in Farm.ServiceProxies
                                          from SPServiceApplicationProxy SvcAppProxy in SvcProxy.ApplicationProxies
                                          where SvcAppProxy is WordServiceApplicationProxy
                                          select SvcAppProxy).FirstOrDefault();

            if (WordServiceAppProxyQry == null)
                return null;
            else
                return WordServiceAppProxyQry as WordServiceApplicationProxy;

        }

        #endregion

        #region SPFile extension methods

        /// <summary>
        /// Converts the file to a byte array in the specified format using Word Automation Services
        /// </summary>
        /// <param name="File"></param>
        /// <param name="OutputFormat">The output format for the file conversion</param>
        /// <returns></returns>
        public static byte[] ConvertSynchronously(this SPFile File, SaveFormat OutputFormat)
        {
            //Check to see that we have a WordServiceApplicationProxy up and running in the farm
            WordServiceApplicationProxy WordSvcAppProxy = SPFarm.Local.WordServiceAppProxy();
            if (WordSvcAppProxy != null)
            {
                //Open the source file in a stream inside a using statement to ensure safe disposal
                using (Stream SourceStream = File.OpenBinaryStream())
                {
                    //Create a new memory stream so we have somewhere to stores the converted file data, again inside a using statement to ensure safe disposal
                    using (MemoryStream OutputStream = new MemoryStream())
                    {
                        //Instantiate a synchronous converter and set key properties
                        SyncConverter Converter = new SyncConverter(WordSvcAppProxy);
                        Converter.UserToken = File.DocumentLibrary.ParentWeb.Site.UserToken;
                        Converter.Settings.UpdateFields = true;
                        Converter.Settings.EmbedFonts = true;
                        Converter.Settings.OutputFormat = OutputFormat;

                        //If the conversion succeeded then return the result as a byte array
                        if (Converter.Convert(SourceStream, OutputStream).Succeeded)
                        {
                            return OutputStream.ToArray();
                        }
                    }
                }
            }

            //Default return value
            return null;
        }

        /// <summary>
        /// Returns true if the specified file has an MS Word file format extension
        /// </summary>
        /// <param name="File"></param>
        /// <returns></returns>
        public static bool IsWordFormat(this SPFile File)
        {
            string Ext = Path.GetExtension(File.Name).ToUpperInvariant();
            return (Ext == ".DOCX") ||
                   (Ext == ".DOCM") ||
                   (Ext == ".DOTX") ||
                   (Ext == ".DOTM") ||
                   (Ext == ".DOC") ||
                   (Ext == ".DOT");
        }

        #endregion

        #region SaveFormat extension methods

        /// <summary>
        /// Returns a Mime type string for the specified SaveFormat
        /// </summary>
        /// <param name="Format"></param>
        /// <returns></returns>
        public static string GetMineType(this SaveFormat Format)
        {
            switch (Format)
            {
                case SaveFormat.Document:
                case SaveFormat.Document97:
                case SaveFormat.DocumentMacroEnabled:
                case SaveFormat.DocumentStrict:
                case SaveFormat.Template:
                case SaveFormat.Template97:
                case SaveFormat.TemplateMacroEnabled:
                    {
                        return System.Web.MimeMapping.GetMimeMapping("a.docx"); //"application/msword";
                    }
                case SaveFormat.MHTML:
                    {
                        return System.Web.MimeMapping.GetMimeMapping("a.mhtml"); //"message/rfc822";
                    }
                case SaveFormat.PDF:
                    {
                        return System.Web.MimeMapping.GetMimeMapping("a.pdf"); // "application/pdf";
                    }
                case SaveFormat.RTF:
                    {
                        return System.Web.MimeMapping.GetMimeMapping("a.rtf"); // "application/rtf";
                    }
                case SaveFormat.XML:
                    {

                        return System.Web.MimeMapping.GetMimeMapping("a.xml"); //"application/xml";
                    }
                case SaveFormat.XPS:
                    {
                        return System.Web.MimeMapping.GetMimeMapping("a.xps"); //"application/vnd.ms-xpsdocument";
                    }

                default:
                    {
                        return "application/unknown";
                    }
            }
        }

        #endregion

    }
}

